{% load i18n %}
{% block headline %}{% endblock %}
{% block main %}
    <form action="." method="post" enctype="multipart/form-data">
        {% csrf_token %}
        {% for error in form.non_field_errors %}
            <p>{{ error|safe }}</p>
        {% endfor %}
        {% for field in form  %}
            {% include "django_libs/partials/form_field.html" with field=field %}
        {% endfor %}
        <p><input type="submit" value="{% trans "Upload" %}"></p>
    </form>
{% endblock %}
