{% extends "admin/change_form.html" %}
{% load static %}

{% block extrahead %}{{ block.super }}
<script type="text/javascript">
    var admin_url = '{% url "admin:index" %}';
    var static_url = '{{ STATIC_URL }}';
    var ddl_ajax_url = '{% url "aps_bom_ddl_ajax" %}';
</script>
{% endblock %}
